{# Render a datetime.

Render absolute timestamp and relative friendly timestamp for it.

User by :py:func:`websauna.system.core.templatecontext.friendly_time`.
#}
<div class="friendly-time">
    {{ time|friendly_time() }}
</div>

<div class="unfriendly-time">
    {{ time|datetime(format=format, target_timezone=timezone, locale="en_US", show_timezone=show_timezone) }}
</div>