{# HTTP 403 - when the view raises pyramid.httpexceptions.HttpForbidden #}

{% extends "site/base.html" %}

{% block content %}
    <div id="forbidden">
        <h1>Forbidden</h1>

        <p>
            Whoopsie. You do not have permission to access this page. 
        </p>
    
        {% if not request.user %}
            <p>
                <a href="{{'login'|route_url}}">Login first?</a>
            </p>
        {% endif %} 
    </div>
{% endblock %}