{# Site title and description in <head>. #}
{% if site_title %}
    <title>{{site_title}}</title>
{% endif %}

{% if site_description %}
    <meta name="description" content="{{site_description}}">
{% endif %}
