{% extends "admin/base_panel.html" %}

{% block panel_title %}
{{ title }} <span class="text-muted">({{count}})</span>
{% endblock panel_title %}

{% block panel_content %}
{% endblock panel_content %}

{% block panel_buttons %}
    {% if controls %}
        {% if request.has_permission('view', context) %}
            <a id="btn-panel-list-{{ model_admin.__name__ }}" class="btn btn-default btn-admin-list" href="{{ request.resource_url(model_admin, 'listing') }}">
                List
            </a>
        {% endif %}


        {% if request.has_permission('add', context) %}
            <a id="btn-panel-add-{{ model_admin.__name__ }}" class="btn btn-default btn-admin-list" href="{{ request.resource_url(model_admin, 'add') }}">
                Add
            </a>
        {% endif %}
    {% endif %}
{% endblock %}
