{% extends "site/base.html" %}

{% block content %}

    {# Navbar menu #}
    {{ request.admin.get_admin_menu_entry().render(request)|safe }}

    {{context|admin_breadcrumbs|safe}}

    {% block admin_content %}
    {% endblock admin_content %}

    {% block crud_content %}
    {% endblock crud_content %}

{% endblock content %}

{% block extra_body_end %}
  <script src="{{ 'websauna.system:static/admin.js'|static_url }}"></script>
{% endblock %}



