{# CSS files on every page of the site.

Load

* :term:`Bootstrap` 3.x

* Websauna core CSS styles (e.g. navbar logout button style fixes)

* :term:`Font Awesome` from :term:`CDN`
#}

{# CDN'ed version #}
{#
<link rel="stylesheet" href="//netdna.bootstrapcdn.com/bootstrap/3.3.4/css/bootstrap.min.css">
 #}
<link rel="stylesheet" href="{{ 'websauna.system:static/bootstrap.min.css'|static_url }}">
<link rel="stylesheet" href="{{ 'websauna.system:static/theme.css'|static_url }}">
<link href="//netdna.bootstrapcdn.com/font-awesome/4.7.0/css/font-awesome.min.css" rel="stylesheet">

{% if request.on_demand_resource_renderer %}
  {% for css_url in request.on_demand_resource_renderer.get_resources("css") %}
    <link rel="stylesheet" href="{{ css_url }}"></link>
  {% endfor %}
{% endif %}

