/**
 * Some helpers dealing with admin interface.
 */

/* global jQuery */

(function($) {
  "use strict";

  $(document).ready(function() {
      // Enable Bootstrap tooltips
      $('[data-toggle="tooltip"]').tooltip();
  });

})(jQuery);