<nav class="navbar navbar-default {% if entry.css_class %}{{ entry.css_class }}{% endif %}">

    <div class="navbar-header">
        <button type="button" class="navbar-toggle" data-toggle="collapse" data-target="#header-{{ entry.id }}">
            <span class="sr-only">Toggle navigation</span>
            <span class="icon-bar icon-bar-red"></span>
            <span class="icon-bar icon-bar-gold"></span>
            <span class="icon-bar icon-bar-green"></span>
        </button>

        {% if entry.label %}
            <a class="navbar-brand" href="{{ entry.get_link(request) }}">
                {{ entry.label }}
            </a>
        {% endif %}
    </div>

    <div class="collapse navbar-collapse" id="header-{{ entry.id }}">
        {% if entry.submenu and entry.submenu.has_items(request) %}
            <ul class="nav navbar-nav">

                {% for subentry in entry.submenu.get_entries() %}
                    {% if subentry.is_enabled(request) %}
                        {{ subentry.render(request)|safe }}
                    {% endif %}
                {% endfor %}
            </ul>
        {% endif %}
    </div>

</nav>