{% extends "site/base.html" %}

{% block content %}
    <div id="waiting-for-activation">
        <h1>Waiting for email activation</h1>

        <p>A confirmation email has been sent to <strong>{{user.email}}</strong>. Please click the link in the email to continue.</p>
    </div>
{% endblock content %}

