{% extends "email/base.html" %}

{% block content %}
    <p>
    Your account on {{site_name}} is pending for email activation. Please click the link below to continue.
    </p>

    <p style="text-align: center; margin: 0 auto">
        <a class="btn-primary" style="font-size: 120%" href="{{link}}">Activate my account</a>
    </p>

    <p>
    The link expires in {{ expiration_hours }} hours.
    </p>

{% endblock content %}