
/* Fix log out form + button in the nav bar to a look like proper link */
.navbar-default form .btn-link {
  padding: 15px;
  background: transparent;
  text-transform: none;
}

.navbar-default form .btn-link:hover {
  text-decoration: none;
}

/* Add some default margin before the footer. Otherwise page text cuts into to footer on pages like Confirm delete */
footer {
  margin-top: 20px;
}

/* For crud/form_button.html - not used in the default UI */
#crud-page-buttons form {
  display: inline-block;
}