{% extends base_template %}

{% block description %}
    <title>{{ title }}</title>
{% endblock description %}

{% block crud_content %}

  <div id="crud-show">
    {% include "crud/resource_controls.html" %}
    {% if title %}
        <h1>{{title}}</h1>
    {% endif %}

    {{form|safe}}
  </div>
{% endblock crud_content %}