<td class="crud-column-{{column.id}}">

    {% with target=column.get_navigate_url(instance, request), value=column.get_value(view, obj)  %}
        {% if target %}
            <a href="{{target}}">
                {{ value }}
            </a>
        {% else %}
            {{ value }}
        {% endif %}
    {% endwith %}
</td>