{% extends "site/base.html" %}

{% block content %}

    <div class="row">
        <div class="col-md-12">
            <h1>Reset</h1>

            <p>Please give the new password below.</p>
        </div>
    </div>

    <div class="row" id="reset-password-form">
        <div class="col-md-12">
            {{ form|safe }}
        </div>
    </div>

{% endblock content %}

