{% extends "site/base.html" %}

{% block content %}

    <div class="row">
        <div class="col-md-12">
            <h1>Sign up to {{site_name}}</h1>
        </div>
    </div>

    <div class="row">
        {% if social_logins %}
            <div class="col-md-6">

                <h2>With social media login</h2>

                <div>
                    {% for social in social_logins  %}
                        {% set template = 'social/' + social + '/register_snippet.html' %}
                        {% include template %}
                    {% endfor %}
                </div>

                <p><!-- --></p>
                <p class="text-center">
                    <small>
                        We never post anything on your wall.
                    </small>
                </p>
            </div>
        {% endif %}


        <div class="col-md-6">
            <h2>With email</h2>

            <div id="sign-up-form">
                {{form|safe}}
            </div>
        </div>
    </div>

{% endblock content %}

