{% extends base_template %}

{% block description %}
    <title>Confirm delete</title>
{% endblock description %}

{% block crud_content %}

    <div class="text-center">
      <h1>Confirm delete</h1>

      <p>
        Are you sure you want to delete <strong>{{ context.get_title() }}</strong>?
      </p>

      {% include "core/interstitial_form.html" %}
    </div>

{% endblock crud_content %}