{# Site footer. #}
<footer>
  <div class="container">
    <div class="row">
      <div class="col-md-12">
        <div class="text-center">
          <small>
              © {{ now().year }} {{ site_author }}
          </small>
        </div>
      </div>
    </div>
  </div>
</footer>