<div id="admin-panel-{{ model_admin.__name__ }}" class="panel panel-default panel-admin">
    {% block panel_heading%}
        <div class="panel-heading clearfix">

            <h3 class="panel-title pull-left">
                {% block panel_title %}
                    {{title}}
                {% endblock %}
            </h3>

            <div class="panel-buttons pull-right">
                {% block panel_buttons %}
                {% endblock panel_buttons %}
            </div>
        </div>
    {% endblock panel_heading %}

  <div class="panel-body">
        {% block panel_content %}
        {% endblock panel_content %}
  </div>

</div>

