{% extends "email/base.html" %}

{# Random sample of UTF-8 text #}
{% block content %}

  <h1 id="heading-sample-email">HTML email template rendering test</h1>

  <p>
    Koulupäivä aloitettiin yhteisesti urheiluhallissa laulamalla Keski-Pohjanmaan laulu ja Laulu Lestijoelle; esilaulajina toimi 7 D-luokka ja säestäjänä mus. op.. Tämän jälkeen oppilaat tutustuivat luokanvalvojan johdolla oheisen ohjelman mukaisiin näyttelyihin.
  </p>

  <p style="text-align: center">
    <a class="btn-primary" href="{{ 'home'|route_url }}">Visit website</a>
  </p>

{% endblock %}