{# Websauna default homepage.

This page is only visible when developing Websauna core or running unit tests against it. Websauna applications always supply their own homepage.
 #}
{% extends "site/base.html" %}

{% block content %}

    <!-- Header -->
    <header>
        <div class="jumbotron text-center">
            <h1 id="front-page-title">{{site_name}}</h1>
            <p class="lead text-center">
                <span class="lead-inner">{{site_tag_line}}</span>
            </p>
        </div>
    </header>

    <section id="sign-up">
        <div class="row">
            <div class="col-lg-12">
                <p class="text-center lead">
                    <div class="alert alert-danger">
                      If you are seeing this text it means you are developing Websauna itself, you are running Websauna unit tests or you have messed up app:main entry point for your application and it loads default Websauna home.
                    </div>
                </p>
            </div>
        </div>
    </section>

    <section id="features">
        <div class="container marketing">

            <div class="row text-center">

                <div class="col-md-4 col-sm-6">

                    <!-- Left -->
                    <div class="entry">
                        <span class="fa-stack fa-4x">
                            <i class="fa fa-circle fa-stack-2x text-primary"></i>
                            <i class="fa fa-heart fa-stack-1x fa-inverse"></i>
                        </span>
                        <h4>You chose Python</h4>
                        <p>Cutting edge programming language</p>
                    </div>

                    <div class="entry">
                        <span class="fa-stack fa-4x">
                            <i class="fa fa-circle fa-stack-2x text-primary"></i>
                            <i class="fa fa-facebook fa-stack-1x fa-inverse"></i>
                        </span>
                        <h4>Easy sign ups</h4>
                        <p>Your uses can sign up using Facebook, Github or other social media accounts.</p>
                    </div>
                </div>

                <!-- Middle -->
                <div class="col-md-4 col-sm-6">

                    <div class="entry">
                        <span class="fa-stack fa-4x">
                            <i class="fa fa-circle fa-stack-2x text-primary"></i>
                            <i class="fa fa-database fa-stack-1x fa-inverse"></i>
                        </span>
                        <h4>Runs on PostgreSQL</h4>
                        <p>Most advanced open source database available today</p>
                    </div>

                    <div class="entry">
                        <span class="fa-stack fa-4x">
                            <i class="fa fa-circle fa-stack-2x text-primary"></i>
                            <i class="fa fa-lock fa-stack-1x fa-inverse"></i>
                        </span>
                        <h4>Secure</h4>
                        <p>Ships with very secure defaults.</p>
                    </div>
                </div>

                <!-- Right -->
                <div class="col-md-4">

                    <div class="entry">
                        <span class="fa-stack fa-4x">
                            <i class="fa fa-circle fa-stack-2x text-primary"></i>
                            <i class="fa fa-globe fa-stack-1x fa-inverse"></i>
                        </span>
                        <h4>Internationalization</h4>
                        <p>Spean many languages through GNU gettext</p>
                    </div>

                    <div class="entry">
                        <span class="fa-stack fa-4x">
                            <i class="fa fa-circle fa-stack-2x text-primary"></i>
                            <i class="fa fa-eye fa-stack-1x fa-inverse"></i>
                        </span>
                        <h4>It's beautiful</h4>
                        <p>Easy theming with industry standard Bootstrap layout and FontAwesome icons</p>
                    </div>
                </div>
            </div>
        </div>
    </section>



{% endblock %}