<td class="crud-column-{{column.id}}">
    <div class="pull-right">

        {% if request.has_permission("view", instance) %}
            <a href="{{ instance|model_url('show') }}" class="btn-crud-listing-show">
                Show
            </a>
        {% endif %}

        {% if request.has_permission("edit", instance) %}
            <a href="{{ instance|model_url('edit') }}" class="btn-crud-listing-edit">
                Edit
            </a>
        {% endif %}

        {% if request.has_permission("delete", instance) %}
            <a href="{{ instance|model_url('delete') }}" class="btn-crud-listing-delete">
                Delete
            </a>
        {% endif %}
    </div>
</td>