{% extends "admin/model_panel.html" %}

{% block panel_content %}
<p>
    {% if latest_user %}
        Latest user: <a id="latest-user-shortcut" href="{{ latest_user_url }}">{{latest_user.friendly_name}}</a>
        (
            <small class=text-muted>
                {{ latest_user.activated_at|friendly_time(source_timezone='UTC') }}
            </small>
        )
    {% endif %}
</p>
{% endblock panel_content %}