{% extends "site/base.html" %}

{% block description %}
<title>Sign in to {{ site_name }}</title>
<meta name="description" content="">
{% endblock %}

{% block content %}

    <div class="row">
        <div class="col-md-12">
            <h1>Login to {{site_name}}</h1>
        </div>
    </div>

    {% include "login/login_base.html" %}

{% endblock content %}

