{% extends "email/base.html" %}

{% block content %}

    <p>
    Hello {{user.friendly_name}},
    </p>

    <p>
    Someone requested resetting your password on {{ site_name }}. If it was you please click the link below.
    </p>

    <p style="text-align: center; margin: 0 auto">
        <a class="btn-primary" style="font-size: 120%" href="{{link}}">Reset password for my account</a>
    </p>

    <p>
    This request expires in {{ expiration_hours }} hours.
    </p>

{% endblock content %}