<td class="crud-column-{{column.id}}">

    {% with target=column.get_navigate_url(instance, request) %}
        {% if target %}
            <a href="{{target}}">

                <div class="friendly-time">
                    {{ column.get_value(view, obj)|friendly_time() }}
                </div>

                <div class="unfriendly-time">
                    <small>
                        {{ column.get_value(view, obj)|datetime(format=column.format, target_timezone=column.timezone, locale="en_US", show_timezone=True) }}
                    </small>
                </div>

            </a>
        {% else %}
            <div class="friendly-time">
                {{ column.get_value(view, obj)|friendly_time() }}
            </div>

            <div class="unfriendly-time">
                <small>
                    {{ column.get_value(view, obj)|datetime(format=column.format, target_timezone=column.timezone, locale="en_US", show_timezone=True) }}
                </small>
            </div>
        {% endif %}
    {% endwith %}
</td>
