{# Resource-specific actions bar rendered at the top of each page #}
<div id="crud-page-controls clearfix">

    <div id="crud-page-buttons" class="pull-right">
        {% for button in resource_buttons %}
            {% if button.is_visible(context, request) %}
                {{ button.render(context, request)|safe }}
            {% endif %}
        {% endfor %}
    </div>

</div>