{# Alternative <html>...</html> framing without header and footer. #}
<!DOCTYPE html>
<html lang="en">

{# Base template which does not use any request context information and is safe to use on error pages #}

<head>

  {% block meta %}
    {# Include or override default <meta> tags #}
    {% include "site/meta.html" %}
  {% endblock meta %}

  {% block css %}
    {# Include or override default CSS head #}
    {% include "site/css.html" %}
  {% endblock %}

  {% block custom_head %}
    {# A block where the template extending this base tempplate can drop in more ``<head>`` elements. #}
  {% endblock %}

</head>

<body>
    {% block body %}
      {#  This block covers everything between ``<body>...</body>`` #}

      <main>

          {# content is the content with margins #}
          <div class="container">
              {% block content %}
              {% endblock content %}
          </div>
      </main>

      {# analytics is tracking JavaScript #}
      {% block analytics %}
          {% include "site/analytics.html" %}
      {% endblock analytics %}

    {% endblock body %}

</body>
</html>
